@@ -15,7 +15,7 @@ https://stackoverflow.com/questions/16138232/is-it-a-good-practice-to-use-try-ex
 """
 import re
 import traceback
-from typing import Any, Collection, Set, List
+from typing import Any, Collection, Optional, Set, List
 
 from mitosheet.types import ColumnHeader
 
@@ -68,14 +68,23 @@ def make_incompatible_merge_headers_error(error_modal: bool=True) -> MitoError:
         error_modal=error_modal
     )
 
-def make_incompatible_merge_key_error(error_modal: bool=True) -> MitoError:
+def make_incompatible_merge_key_error(merge_key_one: Optional[ColumnHeader]=None, merge_key_one_dtype: Optional[str]=None, merge_key_two: Optional[ColumnHeader]=None, merge_key_two_dtype: Optional[str]=None, error_modal: bool=True) -> MitoError:
     """
     Helper function for creating a incompatible_merge_key_error
 
     Occurs when a user merges on two keys that are different types
     """
-    to_fix = f'You cannot merge using keys of different types. Either pick new keys or cast their types.'
+    from mitosheet.column_headers import get_column_header_display
 
+    if merge_key_one is not None and merge_key_two is not None:
+        return MitoError(
+            'incompatible_merge_key_error', 
+            'Incompatible Merge Key Types',
+            f'{get_column_header_display(merge_key_one)} ({merge_key_one_dtype}) and {get_column_header_display(merge_key_two)} ({merge_key_two_dtype}) have different types. Either pick new keys or cast their types.',
+            error_modal=error_modal
+        )
+    
+    to_fix = f'You cannot merge using keys of different types. Either pick new keys or cast their types.'
     return MitoError(
         'incompatible_merge_key_error', 
         'Incompatible Merge Key Types',
@@ -131,6 +140,22 @@ def make_invalid_formula_error(formula: str, to_fix: str=None, error_modal: bool
         error_modal=error_modal
     )
 
+def make_invalid_formula_after_update_error(error_modal: bool=False) -> MitoError:
+    """
+    Helper function for creating a invalid_formula_after_update_error.
+
+    Occurs when a user edits a user tries to resubmit a formula that used to be
+    valid and is no longer valid - which must mean they changed something they 
+    rely on.
+    """
+  
+    return MitoError(
+        'invalid_formula_after_update_error', 
+        'Formula is Now Invalid',
+        'Renaming or deleting other columns in this dataset has made this formula invalid. Please update this formula.',
+        error_modal=error_modal
+    )
+
 
 def make_cast_value_to_type_error(value: str, column_dtype: str, error_modal: bool=True) -> MitoError:
     """
@@ -241,18 +266,27 @@ def make_unsupported_function_error(functions: Set[str], error_modal: bool=True)
         error_modal=error_modal
     )
 
-def make_invalid_column_delete_error(column_headers: Collection[ColumnHeader], dependents: Collection[ColumnHeader]) -> MitoError:
+
+def make_invalid_column_delete_error(column_headers: Collection[ColumnHeader], dependents: Optional[Collection[ColumnHeader]]=None) -> MitoError:
     """
     Helper function for creating a invalid_column_delete_error.
 
     Occurs when:
-    -  the user deletes a column that is referenced by other columns
+    - the user deletes a column that is referenced by other columns
+    - the user deletes a column that does not exist
     """
+    if dependents is not None:
+        header = 'Column Has Dependents'
+        to_fix = f'{(", ").join(map(str, column_headers))} cannot be deleted, as {"they are" if len(column_headers) > 1 else "it is"} referenced in {(", ".join(map(str, dependents)))}. Please remove these references before deleting.'
+    else:
+        header = 'Column Does not exist'
+        to_fix = f'{(", ").join(map(str, column_headers))} cannot be deleted, as not all of these columns exist.'
+   
 
     return MitoError(
         'invalid_column_delete_error',
-        'Column Has Dependents',
-        f'{(", ").join(map(str, column_headers))} cannot be deleted, as {"they are" if len(column_headers) > 1 else "it is"} referenced in {(", ".join(map(str, dependents)))}. Please remove these references before deleting.'
+        header,
+        to_fix
     )
 
 def make_invalid_arguments_error(function: str) -> MitoError:
@@ -396,6 +430,21 @@ def make_no_analysis_error(analysis_id: str, error_modal: bool=True) -> MitoErro
         error_modal=error_modal
     )
 
+def make_invalid_promote_row_to_header(error_modal: bool=True) -> MitoError:
+    """
+    Helper function for creating a invalid_promote_row_to_header.
+
+    Occurs when a user tries to promote a row to header that has duplicated
+    values within it.
+    """
+    to_fix = f'This row has duplicated values in it. As such, making this row a header would make column references ambigious, so we cannot make it a header row.' 
+    
+    return MitoError(
+        'no_analysis_error', 
+        "Cannot create duplicate headers",
+        to_fix,
+        error_modal=error_modal
+    )
 
 ARG_FULL_NAME = {
     'int': 'number',
